@@ -7,6 +7,7 @@
 # (at your option) any later version.
 
 from __future__ import absolute_import
+from __future__ import print_function
 
 import os
 import signal
@@ -144,7 +145,8 @@ def test_clock_measure(fake_time):
     c.stop("sync")
     c.stop("total")
     assert str(c) == (
-        "[total=3.000000, read=1.000000, write=1.000000, sync=1.000000]")
+        "[total=3.000000/1, read=1.000000/1, write=1.000000/1, "
+        "sync=1.000000/1]")
 
 
 def test_clock_measure_multiple(fake_time):
@@ -167,7 +169,8 @@ def test_clock_measure_multiple(fake_time):
     c.stop("sync")
     c.stop("total")
     assert str(c) == (
-        "[total=5.000000, read=2.000000, write=2.000000, sync=1.000000]")
+        "[total=5.000000/1, read=2.000000/2, write=2.000000/2, "
+        "sync=1.000000/1]")
 
 
 def test_clock_running(fake_time):
@@ -177,7 +180,7 @@ def test_clock_running(fake_time):
     c.start("read")
     fake_time.value += 4
     c.stop("read")
-    assert str(c) == "[total=7.000000*, read=4.000000]"
+    assert str(c) == "[total=7.000000/1, read=4.000000/1]"
 
 
 # Inccorrect usage
@@ -201,3 +204,57 @@ def test_clock_stop_missing():
     c = util.Clock()
     with pytest.raises(RuntimeError):
         c.stop("missing")
+
+
+def test_clock_run(fake_time):
+    c = util.Clock()
+    with c.run("total"):
+        with c.run("a"):
+            fake_time.value += 4
+        with c.run("b"):
+            fake_time.value += 3
+    assert str(c) == "[total=7.000000/1, a=4.000000/1, b=3.000000/1]"
+
+
+def test_clock_run_recursive():
+    c = util.Clock()
+    with c.run("started"):
+        with pytest.raises(RuntimeError):
+            with c.run("started"):
+                pass
+
+
+@pytest.mark.benchmark
+def test_benchmark():
+    c = util.Clock()
+    c.start("connection")
+    # We have seen 66,000 requests per single upload with virt-v2v.
+    for i in range(50000):
+        c.start("request")
+        c.start("read")
+        c.stop("read")
+        c.start("write")
+        c.stop("write")
+        c.stop("request")
+    c.stop("connection")
+    print(c)
+
+
+@pytest.mark.parametrize("size,rounded", [
+    (0, 0),
+    (1, 512),
+    (512, 512),
+    (512 + 1, 512 * 2),
+])
+def test_round_up(size, rounded):
+    assert util.round_up(size, 512) == rounded
+
+
+@pytest.mark.parametrize("size,rounded", [
+    (0, 0),
+    (1, 0),
+    (512, 512),
+    (512 + 1, 512),
+])
+def test_round_down(size, rounded):
+    assert util.round_down(size, 512) == rounded
